@@ -194,7 +194,21 @@ class _Datasource(_Base):
         if model_path is not None:
             self.trainers[key]['model_path'] = model_path
 
-        self.dfl.initiate_project_participants(should_spawn_threads=True)
+        for p in self.dfl.project_participants:
+            project_key = p['project_key']
+            datasource_key = p['datasource_key']
+            threads_key = get_threads_key(project_key, datasource_key)
+            '''
+            The first time attach_datasource() we might find 1 previous project for that ds1.
+            So project_participants = [item1]
+            The next time it is called it might also find 1 previous project for ds2
+            So project_participants = [item1, item2]
+            We only want create a thread for items that we haven't done so already
+            '''
+            if threads_key in self.dfl.on_round_threads:
+                continue
+            info = self._make_request('GET', f'/projects/{project_key}')
+            self.dfl.on_round_threads[threads_key] = gevent.spawn(self.dfl.train_and_test_callback, datasource_key, info)
 
 
 class DynamoFL(_Base):
@@ -226,7 +240,7 @@ class DynamoFL(_Base):
         j = json.loads(res)
         if j['event'] == 'client-info':
             self.instance_id = j['data']['id']
-            self.initiate_project_participants(should_fetch_bridges=True, should_spawn_threads=True)
+            # self.initiate_project_participants(should_fetch_bridges=True, should_spawn_threads=True, reset=True)
 
         if j['event'] == 'new-project':
             project_key = j['data']['projectKey']
@@ -257,6 +271,7 @@ class DynamoFL(_Base):
                 threads_key = get_threads_key(project_key, p['datasource_key'])
                 if threads_key in self.on_round_threads:
                     self.on_round_threads[threads_key].kill()
+                    del self.on_round_threads[threads_key]
                 if project_key == p['project_key']:
                     self.on_round_threads[threads_key] = gevent.spawn(self.train_and_test_callback, p['datasource_key'], j['data']['project'])
 
@@ -358,10 +373,17 @@ class DynamoFL(_Base):
             print('Done.')
             print()
 
-    def initiate_project_participants(self, should_fetch_bridges=False, should_spawn_threads=False):
-        self.project_participants = []
+    def initiate_project_participants(self, should_fetch_bridges=False, should_spawn_threads=False, ds=None, reset=False):
+        if reset:
+            self.project_participants = []
+
         if should_fetch_bridges:
-            for ds_key in self.datasources:
+            if ds:
+                datasources = [ds]
+            else:
+                datasources = self.datasources
+                
+            for ds_key in datasources:
                 j = self._make_request('GET', '/bridges', params={'datasourceKey': ds_key})
                 for i in j['data']:
                     self.project_participants.append({
@@ -382,7 +404,7 @@ class DynamoFL(_Base):
                 So project_participants = [item1, item2]
                 We only want create a thread for items that we haven't done so already
                 '''
-                if threads_key in self.dfl.on_round_threads:
+                if threads_key in self.on_round_threads:
                     continue
                 info = self._make_request('GET', f'/projects/{project_key}')
                 self.on_round_threads[threads_key] = gevent.spawn(self.train_and_test_callback, datasource_key, info)    
@@ -409,7 +431,14 @@ class DynamoFL(_Base):
             self._make_request('POST', '/datasources', params=params)
 
         # get current projects of newly attached datasource
-        self.initiate_project_participants(should_fetch_bridges=True)
+        j = self._make_request('GET', '/bridges', params={'datasourceKey': key})
+        for i in j['data']:
+            self.project_participants.append({
+                'project_key': i['projectKey'],
+                'datasource_key': i['datasourceKey'],
+                'trainer_key': i['trainerKey'],
+                'hyper_param_values': i['hyperParamValues']
+            })
 
         ds = _Datasource(self, key)
         self.datasources[key] = ds
